/*
 * This tests a flow for the cloudron owner backs up a cloudron.
 * The backup is validated and restored from.
 */

'use strict';

var AppStore = require('../appstore.js'),
    assert = require('assert'),
    Cloudron = require('../cloudron.js'),
    common = require('../common.js'),
    dnsSync = require('dns-sync'),
    mailer = require('../mailer.js'),
    sleep = require('sleep').sleep;

require('colors');

process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0';
var BOX_VERSION = process.env.BOX_VERSION;

describe('Cloudron backup testing', function () {
    this.timeout(0);

    var appStore = new AppStore('https://api.staging.cloudron.io');

    var owner = common.getOwner();
    var admin = common.getAdmin();
    var cloudron, appId, box, backupInfo;

    it('can login to the store', function () {
        var accessToken = appStore.getAccessToken(owner);
        appStore.setCredentials(owner.password, accessToken);

        var adminAccessToken = appStore.getAccessToken(admin);
        appStore.setAdminCredentials(admin.password, adminAccessToken);
    });

    it('can create a cloudron', function () {
        box = appStore.createCloudron({
            domain: common.cloudronDomain(__filename),
            region: 'sfo1',
            size: '1gb',
            version: BOX_VERSION
        });
        box = appStore.waitForCloudron(box.id);
        cloudron = new Cloudron(box);
    });

    it('can activate the box', function () {
        cloudron.activate(owner);
    });

    it('can login to the box', function () {
        var token = cloudron.getOauthToken(owner);
        cloudron.setCredentials(owner.password, token);
    });

    it('send mail to cloudron user', function (done) {
        mailer.sendMailToCloudronUser(owner.username + '@' + box.domain, done);
    });

    it('can save a sieve script', function (done) {
        cloudron.saveSieveScript(owner, done);
    });

    it('can receive mail', function (done) {
        cloudron.checkMail(owner, done);
    });

    var location = 'test' + (Math.random() * 10000).toFixed();
    it('can install app', function () {
        var manifest = appStore.getManifest(common.TESTAPP_ID, common.TESTAPP_VERSION);
        appId = cloudron.installApp(location, manifest);
    });

    it('can populate the addons', function () {
        cloudron.populateAddons(cloudron.appFqdn(location));
    });

    it('can check the addons', function () {
        cloudron.checkAddons(cloudron.appFqdn(location), owner);
    });

    it('can backup the box', function () {
        backupInfo = cloudron.backup();
        assert.strictEqual(backupInfo.dependsOn.length, 1);
    });

    it('can restore the box', function () {
        appStore.restore(box.id, backupInfo.id);
        box = appStore.waitForCloudron(box.id);
    });

    it('wait for local dns', function () {
        for (var i = 0; i < 50; i++) {
            var ip = dnsSync.resolve(box.domain);
            if (ip === box.ip) return;

             console.log('waiting for local dns to change from %s to %s', ip, box.ip);
             sleep(30);
        }
    });

    it('can login to cloudron', function () {
        var token = cloudron.getOauthToken(owner);
        cloudron.setCredentials(owner.password, token);
    });

    it('retains old mail', function (done) {
        cloudron.checkMail(owner, done);
    });

    it('can check sieve script', function (done) {
        cloudron.checkSieveScript(owner, done);
    });

    it('wait for app', function () {
        cloudron.waitForApp(appId);
    });

    it('can check the addons', function () {
        cloudron.checkAddons(cloudron.appFqdn(location), owner);
    });

    it('can uninstall app', function () {
        cloudron.uninstallApp(appId);
    });

    it('can delete the cloudron', function () {
        appStore.deleteCloudron(box);
    });
});

